<?php
/**
 * @package davidn.de
 * @version 1.11
 */
/*
Plugin Name: E-Paper
Plugin URI: http://www.continent7.com
Description: This Plugin provides a custom post-type e-paper.
Author: David Nellessen (continent7 Bartels & Nellessen GbR)
Version: 1.11
Author URI: http://davidn.de
*/

################################################################################
#
# Filename:         epaper.php
# Type:             php
# Version:          1.11.1
# Last Change:      04.03.2012
# Created:          10.06.2011
# Authors:          David Nellessen (info@davidn.de)
# Organization:     David Nellessen (http://davidn.de)
# Documentation:    http://codex.wordpress.org/Plugin_Resources, 
#                   http://codex.wordpress.org/Post_Types
#
################################ DOCUMENTATION #################################
#
# This is the main file of the epaper plugin for wordpress written
# by David Nellessen.
#
# See readme.txt for more documentation.
#
################################### LICENSE ####################################
#
# Copyright (c) 2012 David Nellessen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
################################## CHANGELOG ###################################
# See readme.txt
#################################### NOTES #####################################
#
# There haven't been any similar epaper or ebook plugins for wordpress yet.
#
# TODO:
#
# IDEAS & future features:
# - IP restriction only if UAM active
# - Support for other plugins (Search, access)
# - Setting for IP-Lifetime (= cron interval (x2))
# - Automatically set Relevanssi to include $frontendfields for indexing
# - A Setting for file permissions
# - Maybe use of WP file system abstraction
# - Relate data folders to posts and not only the other way around. 
#   What if two posts refer to the same dir?
# - Support for Cyrillic alphabet
# - Support for keyword and search parameter to support older flipviewer versions
# 
################################################################################

/*
 * Define some variables
 */
define('EPAPER_PLUGIN_DIR', dirname( __FILE__)  );
define('EPAPER_PLUGIN_URL', trailingslashit( WP_PLUGIN_URL ) . basename( dirname(__FILE__) ) .'/' );   
define('EPAPER_POST_TYPE_NAME', 'epaper'); //internal name for this post-type just like 'post' or 'page'
define('EPAPER_VALIDITY_TIME', 1440); //time in minutes a IP is valide
define('EPAPER_SUBDIR', '/epaper'); //Path where epaper data is stored
define('EPAPER_DEFAULT_PERMISSION', '0777'); //Default Permission when writing files and dirs

global $frontendfields; //these are the custom fields relevant for the frontend (others exists for uploading and path issues)
global $backendfields; //these are the custom fields relevant for the backend (others exists for uploading and path issues)
$frontendfields = $backendfields = array( 
  'epaper_meta_realepaper',
  'epaper_meta_pdf',
  'epaper_meta_pages'
);
$frontendfields_reduced = array(
  'epaper_meta_pages'
);
global $epaper_is_admin; //Caches epaper_is_admin()
$epaper_is_admin = false;

/*
 * Figures out if the current Request has to do with the admin interface of epapers
 * DEPRICATED!!
 */
function epaper_is_admin(){
  global $epaper_is_admin;
  global $post;
  return ($epaper_is_admin or (is_admin() && (
    EPAPER_POST_TYPE_NAME == $_POST['post_type'] or 
    ( !isset($_POST['post_type']) && isset($post) && isset($post->post_type) && $post->post_type == EPAPER_POST_TYPE_NAME ) or 
    ( !isset($_POST['post_type']) && isset($_POST['post']) && get_post_type($_POST['post'])==EPAPER_POST_TYPE_NAME )
    )));
}

/*
 * Start Session if not done yet.
 * 
 * TODO: Better way to start the session?
 * http://www.frank-verhoeven.com/using-session-in-wordpress/
 */
if(!session_id())
session_start();

/*
 * Creates directories for this plugin and stores infos about them in wp_options table
 */
function epaper_create_paths(){
  $wp_uploads = wp_upload_dir(); //http://codex.wordpress.org/Function_Reference/wp_upload_dir
  $dir = get_option( EPAPER_POST_TYPE_NAME . '_dir', $wp_uploads['basedir'] . EPAPER_SUBDIR) ;
  $permission = '' . get_option( EPAPER_POST_TYPE_NAME . '_permission', EPAPER_DEFAULT_PERMISSION) ;
  update_option( EPAPER_POST_TYPE_NAME . '_dir', $dir);
  update_option( EPAPER_POST_TYPE_NAME . '_url', get_option( EPAPER_POST_TYPE_NAME . '_url', $wp_uploads['baseurl'] . EPAPER_SUBDIR) );
  
  if(!is_dir($dir)) mkdir ( $dir, octdec($permission) ); //make 'epaper' dir in wordpress's uploads dir; might want to change rights here!
  chmod($dir, octdec($permission));  
  file_put_contents($dir . '/.htaccess', 'Deny from all'); //create a .htaccess file to protect directory

}

/*
 * Modify the form tag in post.php so that upload works
 */
add_action( 'post_edit_form_tag' , 'epaper_edit_form_tag' );
function epaper_edit_form_tag( ) {
    echo ' enctype="multipart/form-data"';
}

/*
 * Activation hook for this plugin
 * - get permalinks to work when you activate the plugin & do some other activation stuff
 * - register cron job for .htaccess cleanup
 */
register_activation_hook(__FILE__, 'epaper_activation');
add_action('epaper_cron', 'epaper_cleanup'); //epaper_cron is registered as cron job
function epaper_activation() {
  epaper_create_post_type();
  flush_rewrite_rules();
  epaper_create_paths();
  
  //register cron job for .htaccess cleanup; epaper_cron is fired
  wp_schedule_event(time(), 'hourly', 'epaper_cron');
}

/*
 * Deactivation hook for this plugin
 * - clear cron jobs
 */
register_deactivation_hook(__FILE__, 'epaper_deactivation');
function epaper_deactivation() {
  wp_clear_scheduled_hook('epaper_cron');
  flush_rewrite_rules();
  //Remove Data! Might want to comment this out!
  //$dir = get_option( EPAPER_POST_TYPE_NAME . '_dir', $wp_uploads['basedir'] . EPAPER_SUBDIR) ;
  //if(is_dir($dir)) epaper_rrmdir_R($dir);
}

/*
 * Cleanup .htacces file
 * 
 * This function replaces the active .htaccess with the temporary one so that the active one contains data for no longer than twice the cron interval
 */
function epaper_cleanup(){
  //get the list of all directories with epaper data that are protected by this plugin
  $dirs = get_option( 'epaper_dirs', array()); 
  foreach($dirs as $dir){
    $htaccess_filenames = array(
      $dir . '/.htaccess', //active .htacces
      $dir . '/tmp.htaccess' //temporary .htaccess, this will replace the active one on cleanup
    );
    
    //delete active .htacces
    if(is_file($htaccess_filenames[0]) && !unlink($htaccess_filenames[0])){
      //TODO: report error!
    }
    if(is_file($htaccess_filenames[1]) && !rename($htaccess_filenames[1], $htaccess_filenames[0])){
      //TODO: report error!
    }
  } 
}

/*
 * Implement init and admin_init hooks to setup most of the epaper related stuff
 */
add_action( 'init', 'epaper_init' );
function epaper_init(){
  //Internationalization
  load_plugin_textdomain( 'epaper', false, basename(dirname(__FILE__)) .'/languages' );
  //create post-type epaper
  epaper_create_post_type();
  //register stylesheets
  $stylesheeturl = EPAPER_PLUGIN_URL . '/css/frontend.css';
  wp_register_style('epaper-frontendstyle', $stylesheeturl);
  wp_enqueue_style( 'epaper-frontendstyle');
}
add_action( 'admin_init', 'epaper_admin_init', 1 );
function epaper_admin_init(){
  //Internationalization
  load_plugin_textdomain( 'epaper', false, basename(dirname(__FILE__)) .'/languages' );
  epaper_create_metaboxes();
}

/*
 * Function to register scripts for the admin section
 * 
*/
add_action('admin_print_scripts', 'epaper_admin_scripts');
function epaper_admin_scripts() {
  wp_register_script('epaper-backendscript', EPAPER_PLUGIN_URL.'/js/backend.js', array('jquery', 'jquery-ui-tabs'));
  wp_enqueue_script('epaper-backendscript');
}

/*
 * Register styles for the admin section
 * 
*/
add_action('admin_print_styles', 'epaper_admin_styles');
function epaper_admin_styles() {
  $stylesheeturl = EPAPER_PLUGIN_URL . '/css/backend.css';
  wp_register_style('epaper-backendstyle', $stylesheeturl);
  wp_enqueue_style('epaper-backendstyle');
  wp_register_style('epaper-ui-custom-theme', EPAPER_PLUGIN_URL.'/css/jquery-ui-1.8.13.custom.css');
  wp_enqueue_style('epaper-ui-custom-theme');
}  

/*
 * Register a post type for e-paper.
 * 
 * http://codex.wordpress.org/Function_Reference/register_post_type
 */
function epaper_create_post_type() {
  register_post_type(
    EPAPER_POST_TYPE_NAME,
    array(
      'labels' => array(
        'name' => __( 'E-Paper', 'epaper' ),
        'singular_name' =>  __( 'E-Paper', 'epaper'  ),
        'add_new' => __( 'Add New', 'epaper'  ),
        'add_new_item' => __( 'Add New E-Paper', 'epaper'  ),
        'edit_item' => __( 'Edit E-Paper', 'epaper'  ),
        'new_item' => __( 'New E-Paper', 'epaper'  ),
        'view_item' => __( 'View E-Paper', 'epaper'  ),
        'search_items' => __( 'Search E-Paper', 'epaper'  ),
        'not_found' => __( 'No E-Paper found', 'epaper'  ),
        'not_found_in_trash' => __( 'No E-Paper found in Trash', 'epaper'  ),
        'parent_item_colon' => __( 'Parent E-Paper', 'epaper'  ),
        'menu_name' => __( 'E-Paper', 'epaper'  )
      ),
      'description' => __('This is an E-Paper content.', 'epaper' ),
      'publicly_queryable' => true,
      'exclude_from_search' => false,
      'show_ui' => true,
      //'show_in_menu' => NULL,
      'menu_position' => NULL,
      'menu_icon' => EPAPER_PLUGIN_URL . '/epaper-icon.png',
      'capability_type' => 'post',
      //'capabilities' => '' 
      'map_meta_cap' => true,
      'hierarchical' => false,
      //'supports' => array('title', 'editor', 'author', 'thumbnail', 'excerpt', 'comments', 'revisions', 'post-formats'),
      'supports' => array('title', 'editor', 'author', 'thumbnail', 'excerpt'),
      //'register_meta_box_cb' => '',
      'taxonomies' => array('category', 'post_tag'),
      'permalink_epmask' => '',
      //'EP_PERMALINK' => '',
      'has_archive' => true,
      'rewrite' => array(
        'slug' => 'epaper',
      	'with_front' => false,
        //'feeds' => 
        //'pages' => true
      ),
      'query_var' => true, //TODO: check this!
      'can_export' => true,
      //'show_in_nav_menus' => '',
      //'_builtin' => '',  Don't use this!
      //'http://codex.wordpress.org/Function_Reference/register_post_type' => '', Don't use this!
      'public' => true
    )
  );
}

/*
 * Add Metaboxes to epaper post-type
 * 
 * http://codex.wordpress.org/Function_Reference/add_meta_box
 * http://blogs.sitepoint.com/guide-to-wordpress-custom-write-panels/
 */
function epaper_create_metaboxes(){
  add_meta_box( 
  	'epaper_meta_settings', 
    __('E-Paper Settings', 'epaper' ), 
    'epaper_meta_settings_form',
    EPAPER_POST_TYPE_NAME,
    'normal',
    'high',
    NULL );
  add_meta_box( 
  	'epaper_meta_pages', 
    __('E-Paper Content (for Relevanssi search)', 'epaper' ), 
    'epaper_meta_pages_form',
    EPAPER_POST_TYPE_NAME,
    'normal',
    'high',
    NULL );
  add_meta_box( 
  	'epaper_meta_upload', 
    __('Upload epaper data', 'epaper' ), 
    'epaper_meta_upload_form',
    EPAPER_POST_TYPE_NAME,
    'normal',
    'high',
    NULL );
}
/*
 * Form for meta box in add/edit section
 */
function epaper_meta_settings_form(){
  global $post;
  
  //E-Paper URL (real epaper)
  wp_nonce_field( plugin_basename( __FILE__ ) . 'realpaper', 'nonce_epaper_realepaper' );
  $epaper_meta_realepaper = get_post_meta($post->ID, 'epaper_meta_realepaper', TRUE);
  include('templates/admin-meta-realepaper.php');
  
  //PDF-URL 
  wp_nonce_field( plugin_basename( __FILE__ ). 'pdf', 'nonce_epaper_pdf' );
  $epaper_meta_pdf = get_post_meta($post->ID, 'epaper_meta_pdf', TRUE);
  include('templates/admin-meta-pdf.php');
}

/*
 * Form for meta box in add/edit section
 */
function epaper_meta_pages_form(){
  global $post;
  wp_nonce_field( plugin_basename( __FILE__ ). 'pages', 'nonce_epaper_pages' );
  $epaper_meta_pages = get_post_meta($post->ID, 'epaper_meta_pages', TRUE);
  include('templates/admin-meta-pages.php');
}
/*
 * Form for meta box in add/edit section
 */
function epaper_meta_upload_form(){
  global $post;
  wp_nonce_field( plugin_basename( __FILE__ ). 'upload', 'nonce_epaper_upload' );
  //$epaper_meta_upload = get_post_meta($post->ID, 'epaper_meta_upload', TRUE);
  clearstatcache();
  include('templates/admin-meta-upload.php');
}


/*
 * Helper function to find xml file
 * 
 */
function epaper_find_xml_file($dir){
  if ($handle = opendir($dir)) {
    while (false !== ($file = readdir($handle))) {
      if ($file != "." && $file != "..") {
          if(is_dir($dir . '/' . $file)) {
            $file = epaper_find_xml_file($dir . '/' . $file);
            if($file) return $file;
          }
          else{ // Flipviewer Xpress creates a XML File that ends with text_.xml
            if(substr($file, -9) == 'text_.xml') return $dir . '/' . $file;
            elseif(substr($file, -8) == 'text.xml') return $dir . '/' . $file;
          }
      }
    }
    closedir($handle);
  }
  return false;
}

/*
 * Helper function to find PDF file
 * 
 */
function epaper_find_pdf_file($dir, $mustcontain = '__'){
  
  if ($handle = opendir($dir)) {
    while (false !== ($file = readdir($handle))) {
      if ($file != "." && $file != "..") {
          if(!is_dir($dir . '/' . $file)) {
            if(substr($file, -4) == '.pdf' && (empty($mustcontain) or strpos ($file, $mustcontain) !== false) ) return $dir . '/' . $file;
          }
          else{ 
            $file = epaper_find_pdf_file($dir . '/' . $file, $mustcontain);
            if($file) return $file;
          }
      }
    }
    closedir($handle);
  }
  return NULL;
}


/* 
 * When the post is saved, saves our custom data 
 * 
 * Nice Infos:
 * http://codex.wordpress.org/Function_Reference/add_meta_box
 * http://blogs.sitepoint.com/guide-to-wordpress-custom-write-panels/
 * http://www.technokinetics.com/double-saving-with-save-post-hook/
 * 
 */
add_action( 'save_post', 'epaper_save_post' );
function epaper_save_post( $post_id ) {
  
  
  //prevent firing twice
  //http://www.technokinetics.com/double-saving-with-save-post-hook/
  global $epaper_save_post_fired;
  if($epaper_save_post_fired == 1) return true;  
  else $epaper_save_post_fired = 1;
  
  //get the real post id, not the revision id
  //http://wordpress.org/support/topic/save_post-called-repeatedly?replies=6
  $is_post = wp_is_post_revision($post_id);
  if($is_post) $post_id = $is_post;
  
  
  //only react on epaper post-types
  if ( EPAPER_POST_TYPE_NAME != $_POST['post_type'] ) return ;
  
  // verify this came from the our screen and with proper authorization,
  // because save_post can be triggered at other times

  if ( !wp_verify_nonce( $_POST['nonce_epaper_realepaper'], plugin_basename( __FILE__ ) . 'realpaper' ) ){
    //$_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Access denied 2!', 'epaper') . '</p></div>';
    return false;
  }
  if ( !wp_verify_nonce( $_POST['nonce_epaper_pdf'], plugin_basename( __FILE__ ). 'pdf' ) ){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Access denied 3!', 'epaper') . '</p></div>';
    return false;
  }
  if ( !wp_verify_nonce( $_POST['nonce_epaper_pages'], plugin_basename( __FILE__ ) . 'pages' ) ){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Access denied 4!', 'epaper') . '</p></div>';
    return false;
  }
  if ( !wp_verify_nonce( $_POST['nonce_epaper_upload'], plugin_basename( __FILE__ ). 'upload' ) ){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Access denied 5!', 'epaper') . '</p></div>';
    return false;
  }
  
  // Check permissions
  if (!current_user_can('edit_post', $post_id)) return;
  
  //process file upload; returns false if no file uploaded
  if(epaper_process_upload($_FILES['epaper_meta_upload'], $post_id)) {
    $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('Upload completed and processed.', 'epaper') . '</p></div>';
    
    //AUTODETECTION: set realepaper url and pdf url to uploaded dir if we find the correct files
    $uploaddir = get_post_meta($post_id, 'epaper_meta_data_dir', TRUE);
    $basename = basename($uploaddir);
    //guess first try:
    $realepaper = $uploaddir . '/index.html';
    $pdf = $uploaddir . '/' . $basename . '.pdf';
    if(is_file($realepaper) or is_file($pdf)){ //found real epaper data
      if(!is_file($realepaper)) $realepaper = NULL;
      if(!is_file($pdf)) $pdf = NULL;
    }
    elseif(is_dir($uploaddir . '/' . $basename) ){ //if there is a subfolder with the same name, switch to this one (happens when people create archives)
      //guess seccond try:
      $uploaddir .= '/' . $basename;
      $realepaper = $uploaddir . '/index.html';
      $pdf = $uploaddir . '/' . $basename . '.pdf';
      if(is_file($realepaper) or is_file($pdf)){ //found real epaper data
        if(!is_file($realepaper)) $realepaper = NULL;
        if(!is_file($pdf)) $pdf = NULL;
      }
    }
    if(!is_file($pdf)) $pdf = epaper_find_pdf_file(get_post_meta($post_id, 'epaper_meta_data_dir', TRUE));
    if(!is_file($pdf)) $pdf = epaper_find_pdf_file(get_post_meta($post_id, 'epaper_meta_data_dir', TRUE), FALSE);
    
    //Nice trick to transform paths to urls 
    $realepaper = str_replace(WP_CONTENT_DIR, WP_CONTENT_URL, $realepaper);
    $pdf = str_replace(WP_CONTENT_DIR, WP_CONTENT_URL, $pdf);
    
    //find xml-files with E-Paper content
    if(!$xmlfile = epaper_find_xml_file($uploaddir)) 
      $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('E-Paper content could not be recognized: XML-File not found: ', 'epaper')  . '...text_.xml' . '</p></div>';
    elseif(!class_exists("SimpleXMLElement"))
      $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('E-Paper content could not be recognized: No XML-Parser available.', 'epaper')  . '</p></div>';
    else{ //Start processing XML-File
      $xmlobject = simplexml_load_file ( $xmlfile );
      $pagescontent = array();
      $pagenumbers = array();
      if(isset($xmlobject->page) ){
        foreach($xmlobject->page as $page){
          require_once 'forceUTF8.php';
          try {
            $pagenumber = $page->attributes()->pageNumber;
          } catch (Exception $e) {
            $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('There was a page without a pageNumber', 'epaper') . '</p></div>';
            continue;
          }
          array_push($pagenumbers, (int)$pagenumber);
          //protect user input http://stackoverflow.com/questions/1401317/remove-non-utf8-characters-from-string#3521396
          if(isset($page->text)) $pagescontent[(int)$pagenumber] = fixUTF8(print_r((string)$page->text, TRUE)); 
        }
      }
    }
    //fill the the missing pages as empty strings
    for($i=1; $i <= (int)$pagenumber; $i++){
      if(in_array($i, $pagenumbers)) continue;
      $pagescontent[$i] = "";
    }
    ksort ($pagescontent);
    
    //END AUTODETECTION
  }
  
  //store custom fields from backend userinput
  global $backendfields;
  foreach($backendfields as $key){
    $custom_field = $_POST[$key];
    //If no data is entered
    if(is_null($custom_field)) {
      //delete the field. No point saving it.
      delete_post_meta($post_id, $key);
    }
    else
    {
     // update or add field
     update_post_meta($post_id,$key,$custom_field);
    }
  }
  //override epaper_meta_realepaper and epaper_meta_pdf with AUTODETECTION results
  if(!empty($realepaper) && $realepaper != $_POST['epaper_meta_realepaper']){
    $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('E-Paper-URL was replaced from uploaded file: ', 'epaper') . $realepaper . '</p></div>';
    update_post_meta($post_id,'epaper_meta_realepaper', $realepaper);
  }
  if(!empty($pdf) && $pdf != $_POST['epaper_meta_pdf']){
    $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('PDF-URL was replaced from uploaded file: ', 'epaper') . $pdf . '</p></div>';
    update_post_meta($post_id,'epaper_meta_pdf', $pdf);
  }
  if(!empty($pagescontent) && $pagescontent != $_POST['epaper_meta_pages']){
    $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('E-Paper Content was replaced from uploaded file: ', 'epaper') . $xmlfile . '</p></div>';
    update_post_meta($post_id,'epaper_meta_pages', $pagescontent);
  }
  
  //if everything is alright, grant access for current user
  epaper_grant_access_to_post($post_id, empty($_SERVER['REMOTE_ADDR']) ? $_SERVER['HTTP_CLIENT_IP'] : $_SERVER['REMOTE_ADDR']);
  
  
  return $post_id;
}

/*
 * functions that processes the uploaded file
 * 
 * Uses Wordpress core methods to unzup files (unzip_file)
 * 
 */
function epaper_process_upload($fileobject, $post_id){
  //Permission to set for files and dirs
  $permission = '' . get_option( EPAPER_POST_TYPE_NAME . '_permission', EPAPER_DEFAULT_PERMISSION) ;
  
  
  //return if no file uploaded
  if(!isset($fileobject) or !isset($fileobject['error'])){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('No File uploaded.', 'epaper') . '</p></div>';
    return false;
  } 
  if($fileobject['error'] == 4) { //no file uploaded
    $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('No file uploaded.', 'epaper')  . '</p></div>';
    return false;
  } 
  elseif($fileobject['error'] != 0) { //other errors
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('There were errors uploading the file. Error code:', 'epaper')  .$fileobject['error'] . '.</p></div>';
    return false;
  } 
  
  //upload file
  $dir = get_option( EPAPER_POST_TYPE_NAME . '_dir') ;
  $file = $dir . '/' . basename($fileobject['name']);
  $destination = $dir . '/' . basename($fileobject['name'], '.zip');
  if (!move_uploaded_file($fileobject['tmp_name'], $file)){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Could not copy uploaded file.', 'epaper')  . '</p></div>';
    return false;
  } 
  //Set permission
  if(!epaper_chmod_R ($file, $permission)){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Permissions could not be set for: ', 'epaper')  . $file . '</p></div>';
  }
  
  //Work with only FS_METHOD = 'direct' since we get a lot of problems otherwise
  if(defined('FS_METHOD') && FS_METHOD != 'direct') {
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: This Plugin support FS_METHOD = direct only.', 'epaper')  . '</p></div>';
    return false;
  }
  elseif (!defined('FS_METHOD')) define('FS_METHOD', 'direct');
  
  //Get Filesystem credentials
  if (get_filesystem_method(array(), $dir) != 'direct' && false === ($credentials = request_filesystem_credentials($url, '', false,  $dir, null) ) ) {
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: No filesystem credentials. You might have typed in wrong connection details.', 'epaper')  . '</p></div>';
  	return; // stop processing here
  }
  
  //unzip archive
  if ( ! WP_Filesystem($credentials, $dir) ) {
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Could not open .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
    return false;
  }
  global $wp_filesystem;
  
  //$file_upload = new File_Upload_Upgrader('pluginzip', 'package');
  
  //Unzip File
  $result = unzip_file($file, $destination);
  if(is_wp_error($result)){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p> '. __('Error uploading file: ', 'epaper')  . (esc_html( $result->get_error_message() )) . '</p></div>';
    return false;
  } 
  //Set permission
  if(!epaper_chmod_R ($destination, $permission)){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Permissions could not be set for: ', 'epaper')  . $destination . '</p></div>';
  }

  //write .htaccess file into $destination folder
  $htaccess_filename =  $destination . '/.htaccess';
  $htaccess_content = "<Files tmp.htaccess>\nOrder allow,deny\nDeny from all\n</Files>\nOrder Deny,Allow\nDeny from all\nAllow from 000.000.000.000";
  if (!$handle = @fopen($htaccess_filename, 'w')) { 
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Could not open .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
    return false;
  }
  if (fwrite($handle, $htaccess_content) === FALSE) {
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Could not write .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
    return false;
  }
  fclose($handle);
  //Set permission
  if(!epaper_chmod_R ($htaccess_filename, $permission)){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error uploading file: Permissions could not be set for: ', 'epaper')  . $htaccess_filename . '</p></div>';
  }
  
  //delete archive
  if(!unlink ( $file )){
    $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error finishing file upload: Could not unlink uploaded file:', 'epaper') . $file . '</p></div>';
    return false;
  } 
  
  //if everything is fine, add this dir to the list of occupied dirs and set post_meta infos about the dir
  $dirs = get_option( 'epaper_dirs',  array());
  if(!in_array($destination, $dirs))
    $dirs[] = $destination;
  update_option( 'epaper_dirs',  $dirs);
  update_post_meta($post_id, 'epaper_meta_upload', $file);
  update_post_meta($post_id, 'epaper_meta_data_dir', $destination);

  return true;
}

/*
 * Add notices and errors that happened while saving the post
 * 
 * Idea: http://wordpress.stackexchange.com/questions/5102/add-validation-and-error-handling-when-saving-custom-fields/19980#19980
 */
add_action( 'admin_notices', 'epaper_admin_notices' );
function epaper_admin_notices(){
  if(!empty($_SESSION['epaper_admin_notices'])) print  $_SESSION['epaper_admin_notices'];
  unset ($_SESSION['epaper_admin_notices']);
}

/*
 * function to grant access to an epaper directory for a certain IP based on directory
 */
function _epaper_grant_access_to_dir($dir, $ip, $suppress_errors){
  $htaccess_filenames = array(
    $dir . '/.htaccess', //active .htacces
    $dir . '/tmp.htaccess' //temporary .htaccess, this will replace the active one on cleanup
  );
  
  foreach($htaccess_filenames as $htaccess_filename){
    //create file if not exists
    if(!is_file($htaccess_filename)){
      $htaccess_content = "<Files tmp.htaccess>\nOrder allow,deny\nDeny from all\n</Files>\nOrder Deny,Allow\nDeny from all\nAllow from 000.000.000.000";
      if (!$handle = fopen($htaccess_filename, 'w')) { 
        if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error securing data: Could not open .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
        return false;
      }
      if (fwrite($handle, $htaccess_content) === FALSE) {
        if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error securing data: Could not write .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
        return false;
      }
      fclose($handle);
    }
    //write file
    $htaccess_content = "\nAllow from $ip";
    if (!$handle = fopen($htaccess_filename, 'a')) { 
        if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error securing data: Could not open .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
        return false;
      }
      if (fwrite($handle, $htaccess_content) === FALSE) {
        if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error securing data: Could not write .htaccess file to secure epaper data.', 'epaper')  . '</p></div>';
        return false;
      }
    fclose($handle);
  }
}

/*
 * function to grant access to an epaper directory for a certain IP based on post_id
 */
function epaper_grant_access_to_post($post_id, $ip, $suppress_errors = false){
  //check if IP-Adress is valid
  if(ip2long ( $ip) === false or ip2long ( $ip) == -1) {
    if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="error"><p>' . __('Error: Could not detect IP adrress : ', 'epaper')  . $ip . '. Post ID: '.$post_id.'</p></div>';
    return false;
  }
  //load directory from post_meta table
  $dir = get_post_meta($post_id, 'epaper_meta_data_dir', TRUE);
  if(!is_dir($dir)){
    //if(!$suppress_errors) $_SESSION['epaper_admin_notices'] .= '<div class="updated"><p>' . __('Notice: There is no directory to set permissions ', 'epaper')  . $dir . '. Post ID: '.$post_id.'</p></div>';
    return false;
  }
  //update .htaccess file
  return _epaper_grant_access_to_dir($dir, $ip, $suppress_errors);
}

/*
 * Render the_content for the epaper post-type
 * 
 * http://wordpress.org/support/topic/snippets-not-created-from-custom-fields?replies=32
 */
add_filter('the_content', 'epaper_the_content');
function epaper_the_content($content) {
  
  /*
  //To increase performance by avoiding is_the_content_called_by_relevanssi() on non epaper contents
  global $post;
  if(isset($post) && isset($post->post_type) && $post->post_type != EPAPER_POST_TYPE_NAME) return $content;

  //if the_content was triggered by relevanssi, we append the content of all relevant custom fields so that relevanssi includes their content (inlcuding the real epaper content if specified)
  if($posttmp = is_the_content_called_by_relevanssi()){
    // Fields to load content from
    global $frontendfields; 
    foreach($frontendfields as $field){
      //pay attention to $single (http://codex.wordpress.org/Function_Reference/get_post_meta)
      //die(print_r(get_post_meta($post->ID, $field, false), TRUE));
      $field_value = get_post_meta($post->ID, $field, TRUE);
      $content .= ' '. ( is_array($field_value) ? implode(' ', $field_value) : $field_value );
    } 
    $post = $posttmp;
  }
  else{
    global $post;
    $body = $content;
    $content = '';
  }  
  */
  
  global $post;
  
  if ( $post->post_type == EPAPER_POST_TYPE_NAME ) {
    if(!is_object($post)) return NULL;
    
    //check if UAM is installed and let him check permission
    global $userAccessManager;
    $access = true;
    if (isset($userAccessManager)) {
        $uamAccessHandler = $userAccessManager->getAccessHandler();
        $access = $uamAccessHandler -> checkObjectAccess($post->post_type, $post->ID);
    }
    //if UAM says access denied, do not continue!
    if(!$access) return $content;
    
    //If access granted, grant access to epaper data dir, but only if single (perfomance issue, the user should open the post first anyway)
    if(is_singular(EPAPER_POST_TYPE_NAME)) epaper_grant_access_to_post($post->ID, empty($_SERVER['REMOTE_ADDR']) ? $_SERVER['HTTP_CLIENT_IP'] : $_SERVER['REMOTE_ADDR'] , true);
    
    //Get meta values & build template; only if is_single for performance reason
    if(is_singular(EPAPER_POST_TYPE_NAME)){
      $epaper = get_post_meta($post->ID, 'epaper_meta_realepaper', TRUE);
      $pdf = get_post_meta($post->ID, 'epaper_meta_pdf', TRUE);
      
      //alter the url to realepaper and add searchterm and or pagenumber
      $referrer = preg_replace('@(http|https)://@', '', stripslashes(urldecode($_SERVER['HTTP_REFERER'])));
      $args     = explode('?', $referrer);
      $query    = array();
      if ( count( $args ) > 1 )
    	parse_str( $args[1], $query );
      if (substr($referrer, 0, strlen($_SERVER['SERVER_NAME'])) == $_SERVER['SERVER_NAME'] && (isset($query['s']) || strpos($referrer, '/search/') !== false)) {
        //Local search
        if(!empty($query['s'])){
          //$querystring_encoded =  str_replace('+', '%20', urlencode(utf8_decode(str_replace('\\"', '', $query['s']))));
          $querystring_encoded =  str_replace('+', '%20', urlencode(str_replace('\\"', '', $query['s'])));
    	    $epaper .= (strpos($epaper, '?') === false) ?  '?keyword=' .  $querystring_encoded : '&keyword=' . $querystring_encoded ;
        }
        //create message
        $refferer_is_search = true;
        $pagenumber = epaper_get_page_number($post->ID, $query);
        if($pagenumber === false) $pagenumber = 0;
        $epaper .= '&pn='.$pagenumber;
        $searchterm = str_replace('\\"', '', $query['s']);
      }
      else $refferer_is_search = NULL;
    
      //get Thumbnail - $thumbnail used in template
      $attr = array(
  			'class'	=> 'attachment-medium',
  			'alt'	=> esc_attr(trim(strip_tags( $post->post_excerpt ))),
  			'title'	=> esc_attr(trim(strip_tags( $post->post_title ))),
  	  );
      $thumbnail = get_the_post_thumbnail( $post->ID, 'medium', $attr );
      
      //Render output with template
      ob_start ( );
      $themefile = get_stylesheet_directory() . '/epaper-the-content.php';
      if(is_file($themefile)) include($themefile);
      else include('templates/epaper-the-content.php');
  	  $content .= ob_get_contents();
      ob_end_clean();
    }
  } 
  
  return $content;
}

/*
 * Function to detect the page where the performed search hits best
 * 
 * only waorks with relevanssi
 */
function epaper_get_page_number($postid, $query){
  $steps = 6;
  if(!function_exists('relevanssi_fetch_excerpt') or empty($postid) or !isset($query['s'])) return false;
  $excerpt = strip_tags(relevanssi_fetch_excerpt($postid, trim(stripslashes(mb_strtolower($query['s'])))));
  if(empty($excerpt)) return false;
  $excerpt_length= strlen($excerpt);
  if($excerpt_length==0) return false;
  
  //get array with pages' contents
  $pages = get_post_meta($postid, 'epaper_meta_pages', TRUE);
  if(!is_array($pages)) return false;
  
  //cutting size | amoung of characters to reduce each iteration
  $cut_size = ceil(($excerpt_length/$steps)/2);
  //check if parts of the excerpt are contained in the pages' contents
  for($i=0; $i < $steps; $i++){
    $string_to_check = substr($excerpt, $i * $cut_size, -1 * $cut_size * $i );
    if($string_to_check === false) continue;
    
    //check each page
    $pagenumber = 0;
    foreach($pages as $pagecontent){ 
      $pagenumber++;
      if(stripos(strip_tags($pagecontent), $string_to_check)) return $pagenumber;
    }
  }
  return false;
}


/*
 * Build Content for Relevanssi excerpt. Does not respect UAM since we want to show hits anyway!
 * 
 * http://wordpress.org/support/topic/snippets-not-created-from-custom-fields?replies=32
 */
add_filter('relevanssi_excerpt_content', 'epaper_relevanssi_excerpt_content', 10, 3);
function epaper_relevanssi_excerpt_content($content, $post = '', $query = ''){
  //only perform action with epaper post-type
  if ( $post->post_type != EPAPER_POST_TYPE_NAME ) return $content;
  
  global $frontendfields_reduced;
  foreach($frontendfields_reduced as $field){
    //pay attention to $single (http://codex.wordpress.org/Function_Reference/get_post_meta)
    $field_value = get_post_meta($post->ID, $field, TRUE);
    $content .= ' '. ( is_array($field_value) ? implode(' ', $field_value) : $field_value );
  } 
  return $content;
}

/*
 * Check if the_content was triggered by relevanssi
 * DEPRICATED
 * 
 * http://wordpress.org/support/topic/snippets-not-created-from-custom-fields?replies=11#post-2168264
 */
function is_the_content_called_by_relevanssi(){
  //if relevanssi_init() does not exists, there is no relevanssi and relevanssi cannot have triggered the_contet
  if(!function_exists('relevanssi_init')) return false;
  
  /*
   * If we find a function call 'relevanssi_do_excerpt' in backtrace, the_content is triggered by relevanssis to create the excerpt from the_content
   * 
   * expect not more than 10 the_content hooks
   * 21 calculates as follows:
   * apply_filters() + 10 * (call_user_func_array() + callbackfunction()) = 21 
   * you might want to increase this. Remember it slows down you script!
   */
  //limit trace depth if php version is newer than 5.4.0
  if (strnatcmp(phpversion(),'5.4.0') >= 0) 
    $trace = debug_backtrace(DEBUG_BACKTRACE_PROVIDE_OBJECT, 21);
  else
    $trace = debug_backtrace();
  //search for relevanssi_do_excerpt() in backtrace
  for($i = 0; $i<min(array(21,count($trace))); $i++){ 
    //if we find relevanssi_do_excerpt in the stack return the $post object, which we need
    if('relevanssi_do_excerpt' == $trace[$i]['function']) return $trace[$i]['args'][0];
  }
  return FALSE; //nothing found
}

/*
 * Filesystem function to remove dirs recusivly
 */
function epaper_rrmdir_R($dir) {
   if (is_dir($dir)) {
     $files = scandir($dir);
     foreach ($files as $file) {
       if ($file != '.' && $file != '..') {
         if (filetype($dir . '/' . $file) == 'dir') epaper_rrmdir_R($dir . '/' . $file); else unlink($dir . '/' . $file);
       }
     }
     reset($files);
     rmdir($dir);
   }
 } 
 
/*
 * Filesystem function to chmod recusivly
 */
function epaper_chmod_R($path, $filemode) {
    if (!is_dir($path))
       return @chmod($path, octdec($filemode));

    $handle = opendir($path);
    while ($file = readdir($handle)) {
      if($file != '.' && $file != '..') {
        $fullpath = $path.'/'.$file;
        if(!is_dir($fullpath)) {
          if (!@chmod($fullpath, octdec($filemode))){
             return FALSE;
          }
        } else {
          if (!epaper_chmod_R($fullpath, $filemode)){
             return FALSE;
          }
        }
      }
    }
    closedir($handle);
    if(@chmod($path, octdec($filemode)))
      return TRUE;
    else
      return FALSE;
}
